/*  DeltaSMT: Delta-debugger for Satisfiablity Modulo Theories (SMT) benchmarks.
 *  Copyright (C) 2009  Robert Daniel Brummayer
 *
 *  This file is part of DeltaSMT.
 *
 *  DeltaSMT is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DeltaSMT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.*;

public class SMTTypeManager
{
  /* primitive types */
  public static final BooleanType booleanType = new BooleanType();

  public static final IntegerType integerType = new IntegerType();

  public static final RealType realType = new RealType();

  /* AX Types */
  public static final UType indexType = new UType ("Index");

  public static final UType elementType = new UType ("Element");

  public static final ArrayType arrayTypeAX = new ArrayType (indexType, 
                                                             elementType);

  public static final ArrayType arrayTypeIntInt = new ArrayType (integerType,
                                                                 integerType);

  protected HashMap<Integer,BVType> BVTypes;

  protected HashMap<String,BVArrayType> BVArrayTypes;

  protected HashMap<String,UType> uTypes;


  public SMTTypeManager () {
    this.BVTypes = new HashMap<Integer, BVType>();
    this.BVArrayTypes = new HashMap<String, BVArrayType>();
    this.uTypes = new HashMap<String,UType>();
  }

  public BVType BVType (int width) {
    BVType result;

    assert (width > 0);

    result = BVTypes.get(width);
    if (result == null) {
      result = new BVType (width);
      BVTypes.put (width, result);
    }
    return result;
  }

  public BVArrayType BVArrayType (int indexWidth, int elementWidth) {
    BVArrayType result;
    BVType indexType, elementType;
    String hashKey;

    assert (indexWidth > 0);
    assert (elementWidth > 0);

    hashKey = indexWidth + " " + elementWidth;
    result = BVArrayTypes.get(hashKey);
    if (result == null) {
      indexType = this.BVType(indexWidth);
      elementType = this.BVType(elementWidth);
      result = new BVArrayType (indexType, elementType);
      BVArrayTypes.put (hashKey, result);
    }
    return result;
  }

  public UType uType (String name){
    UType result;
    assert (name != null);
    if ((result = uTypes.get(name)) == null) {
      result = new UType(name);
      uTypes.put (name, result);
    }
    return result;
  }

}
