/*  DeltaSMT: Delta-debugger for Satisfiablity Modulo Theories (SMT) benchmarks.
 *  Copyright (C) 2009  Robert Daniel Brummayer
 *
 *  This file is part of DeltaSMT.
 *
 *  DeltaSMT is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DeltaSMT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.*;

public enum TokenKind
{
  NONE("none"), 
  COLON(":"),
  IDENT("identifier"),
  NUMERAL("numeral"),
  USER_VALUE("user value"),
  STRING("string"),
  LPAR("("),
  RPAR(")"),
  LBRACK("["),
  RBRACK("]"),
  VAR("var"),// ?var
  FVAR("fvar"), // $var
  ATTRIB("attribute"), // :attrib
  ARITH_SYMB("arithmetic symbol"),
  TRUE("true"),
  FALSE("false"),
  NOT("not"), 
  BVNOT("bvnot"),
  BVNEG("bvneg"),
  TILDE("~"),
  EXTRACT("extract"),
  REPEAT("repeat"),
  ZERO_EXTEND("zero_extend"),
  SIGN_EXTEND("sign_extend"),
  ROTATE_LEFT("rotate_left"),
  ROTATE_RIGHT("rotate_right"),
  AND("and"),
  OR("or"),
  XOR("xor"),
  IFF("iff"),
  BVAND("bvand"),
  BVNAND("bvnand"),
  BVOR("bvor"),
  BVNOR("bvnor"),
  BVXOR("bvxor"),
  BVXNOR("bvxnor"),
  BVADD("bvadd"),
  BVMUL("bvmul"),
  BVCOMP("bvcomp"),
  EQ("="),
  IMPLIES("implies"),
  BVULT("bvult"),
  BVULE("bvule"),
  BVUGT("bvugt"),
  BVUGE("bvuge"),
  BVSLT("bvslt"),
  BVSLE("bvsle"),
  BVSGT("bvsgt"),
  BVSGE("bvsge"),
  BVSHL("bvshl"),
  BVLSHR("bvlshr"),
  BVASHR("bvashr"),
  BVSUB("bvsub"),
  BVUDIV("bvudiv"),
  BVUREM("bvurem"),
  BVSDIV("bvsdiv"),
  BVSREM("bvsrem"),
  BVSMOD("bvsmod"),
  CONCAT("concat"),
  LT("<"),
  LE("<="),
  GT(">"),
  GE(">="),
  PLUS("+"),
  MINUS("-"),
  TIMES("*"),
  DIV("/"),
  SELECT("select"),
  ITE("ite"), // if-then-else on terms
  IF_THEN_ELSE("if_then_else"), // if-then-else on boolean operands 
  STORE("store"),
  /* n-ary operator */
  DISTINCT("distinct"),
  /* begin of attributes */
  A_LOGIC(":logic"),
  A_ASSUMPTION(":assumption"),
  A_FORMULA(":formula"),
  A_STATUS(":status"),
  A_EXTRASORTS(":extrasorts"),
  A_EXTRAFUNS(":extrafuns"),
  A_EXTRAPREDS(":extrapreds"),
  A_SOURCE(":source"),
  A_DIFFICULTY(":difficulty"),
  A_CATEGORY(":category"),
  A_NOTES(":notes"),
  A_SORTS(":sorts"),
  A_FUNS(":funs"),
  A_PREDS(":prds"),
  A_DEFINITION(":definition"),
  A_AXIOMS(":axioms"),
  A_THEORY(":theory"),
  A_LANGUAGE(":language"),
  A_EXTENSIONS(":extensions"),
  /* end of attributes */
  EXISTS("exists"),
  FORALL("forall"),
  LET("let"),
  FLET("flet"),
  THEORY("theory"),
  LOGIC("logic"),
  BENCHMARK("benchmark"),
  SAT("sat"),
  UNSAT("unsat"),
  UNKNOWN("unknown"),
  EOF("end of file");

  protected String string;

  TokenKind(String string){
    this.string = string;
  }

  public String getString(){
    return this.string;

  }

  public static boolean isBenchAttribute (TokenKind kind){
    assert (kind != null);

    return kind.ordinal() >= TokenKind.A_LOGIC.ordinal() && 
           kind.ordinal() <= TokenKind.A_NOTES.ordinal();
  }

  public final static HashMap <String, TokenKind> arithStringToTokenKind;

  static {
    arithStringToTokenKind = new HashMap<String, TokenKind>();
    arithStringToTokenKind.put ("=", TokenKind.EQ);
    arithStringToTokenKind.put ("<", TokenKind.LT);
    arithStringToTokenKind.put ("<=", TokenKind.LE);
    arithStringToTokenKind.put (">", TokenKind.GT);
    arithStringToTokenKind.put (">=", TokenKind.GE);
    arithStringToTokenKind.put ("~", TokenKind.TILDE);
    arithStringToTokenKind.put ("+", TokenKind.PLUS);
    arithStringToTokenKind.put ("-", TokenKind.MINUS);
    arithStringToTokenKind.put ("*", TokenKind.TIMES);
    arithStringToTokenKind.put ("/", TokenKind.DIV);
  }


}
